unit UMainSys;

interface

uses
	Windows, USmDx3D, DDUtils, DDVer, USmUtil, DirectDraw;

const
	MAX_IMAGE     = 3;
	MAX_PARTICLE  = 1000;
	PARTICLE_STEP = 11;

type
	TGameOption = record
		Handle        : THandle;
		Width         : Integer;
		Height        : Integer;
		Depth         : Integer;
		UseFullScreen : Boolean;
	end;

	TMainSystem = packed class
		constructor Create( Handle : THandle );
		procedure   Free;

	private

	public
		BMPImage   : array[0..Pred( MAX_IMAGE   )] of II_DirectDrawSurface;

		function  MainProc : Boolean;

	end;

var
	MainSystem  : TMainSystem;
	SmDirect3D  : TSmDirect3D;
	GameOption  : TGameOption;
	GameCursor  : TPoint;
	GameKey     : Word;
	ResourceDLL : integer = 0;

implementation

var
	ColorMode   : Word;
	JpegSize    : TPoint;

{ TMainSystem }

constructor TMainSystem.Create( Handle : THandle );
var
	hBM : HBITMAP;
	DDPixelFormat : TDDPixelFormat;
begin
	Inherited Create;

	// Ķ ʱȭ
	ZeroMemory( @BMPImage,  sizeof( BMPImage  ) );

	GameOption.Handle := Handle;

	// Direct X ʱȭ
	with GameOption do
		SmDirect3D    := TSmDirect3D.Create3D( Handle, Width, Height, Depth, UseFullScreen );

	// Image ε
	BMPImage[0] := DDCreateSurface( SmDirect3D.DirectDraw, GameOption.Width, GameOption.Height, TRUE );
	DDClearDevice( BMPImage[0], $0000 );

	DDLoadToJPEGImage( SmDirect3D.DirectDraw, BMPImage[1], JpegSize, 'Status' );

	ZeroMemory( @DDPixelFormat, sizeof( DDPixelFormat ) );
	DDPixelFormat.dwSize := sizeof( DDPixelFormat );
	BMPImage[1].GetPixelFormat( DDPixelFormat );

	if DDPixelFormat.dwRBitMask = $0000F800 then
		ColorMode := 16
	else
	if DDPixelFormat.dwRBitMask = $00007C00 then
		ColorMode := 15
	else
		ColorMode := 16
end;

procedure   TMainSystem.Free;
var
	i : Integer;
begin
	for i := 0 to Pred( MAX_IMAGE ) do begin
		if Assigned( BMPImage[i] ) then
			BMPImage[i] := nil;
	end;

	SmDirect3D.Free;

	Inherited;
end;

function    TMainSystem.MainProc : Boolean;
const
	Count    : Integer = 0;
	M_X      : Integer = 0;
	M_Y      : Integer = 0;
	R_Shift  : Word    = $FFFF;
	G_Shift  : Word    = $FFFF;
var
	x, y     : Integer;
	R, G, B  : WORD;
	pDest    : PWORD;
	TempRect : TRect;
	DestLock : TT_DDSurfaceDesc;
begin
	if R_Shift = $FFFF then begin
		if ColorMode = 15 then begin
			R_Shift := 10;
			G_Shift :=  5;
		end
		else begin
			R_Shift := 11;
			G_Shift :=  6;
		end;
	end;

	Inc( Count );

	// Atennuation
	if Count mod 1 = 0 then begin
		ZeroMemory( @DestLock, sizeof( DestLock ) );
		DestLock.dwSize := sizeof( DestLock );

		BMPImage[0].Lock( nil, DestLock, DDLOCK_SURFACEMEMORYPTR or DDLOCK_WAIT, 0 );

		for y := 0 to Pred( GameOption.Height ) do begin
			pDest := PWORD( DWORD( DestLock.lpSurface ) + y * DestLock.lPitch );
			for x := 0 to Pred( GameOption.Width ) do begin
				R := ( pDest^ shr R_Shift ) and $001F;
				G := ( pDest^ shr G_Shift ) and $001F;
				B := ( pDest^             ) and $001F;
				if R > 0 then Dec( R ) else R := 0;
				if G > 0 then Dec( G ) else G := 0;
				if B > 0 then Dec( B ) else B := 0;
				pDest^ := ( R shl R_Shift ) or ( G shl G_Shift ) or B;
				Inc( pDest );
			end;
		end;

		BMPImage[0].Unlock( nil );
	end;

	M_X := ( M_X + 2 ) mod 540;
	M_Y := ( GameOption.Height - JpegSize.y ) div 2;

	if ( M_X < 440 ) then begin
		TempRect := Bounds( 0, 0, 200, 410 );
		BMPImage[0].BltFast( M_X, M_Y, BMPImage[1], @TempRect, 1 );

		TempRect := Bounds( 210, 0, 190, 410 );
		BMPImage[0].BltFast( 440-M_X, M_Y, BMPImage[1], @TempRect, 1 );
	end;

	SmDirect3D.DirectDrawSurface.BltFast( 0, 0, BMPImage[0], nil, 0 );

	SmDirect3D.FlipPage;

	Result := ( HiByte( GetAsyncKeyState( VK_ESCAPE ) ) = 0 );
end;

end.

